function [ P,Zi,Zf,trameCouranteZ,tag] = Peak2PartialLp( A, F,deltaF, dureeActif, Zi, Zf,  trameCouranteZ, tag)
% A : amplitude of the peaks
% F : frequency of the peaks (in Hz)
%Zf : matrice de sauvegarde des frquences des partiels
%Zi : 1re colonne: numero du partiel (si =0 alors ligne disponible)
%     2me colonne: compteur de dure d'activation 
%     3me colonne: nombre de donnes anciennement sauvegardes disponibles
%Chaque ligne de Zf et Zi correspond  un mme partiel
%dureeActif correspond au nombre de trame pendant lequel un partiel qui ne trouve pas de pic valable,reste actif.
%trameCouranteZ correspond  la trame courante dans les 2 matrices de sauvegardes Zf,Zi 
%dureeActif doit tre infrieur  size(Zf,2)-1; 
%%tag : numero du dernier partiel
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.

if nargin<2, [A,F]=tone2peaks();end
if nargin<3, deltaF=40;end
if nargin<4,dureeActif=7;end
if nargin<8, Zi=zeros(10,3);Zf=zeros(10, 20);trameCouranteZ=1;tag=1;end

    
nbTrames= size(F, 2);
nbPeaks= size(F, 1);
n=size(Zf,2);

P=zeros(nbPeaks,nbTrames);%matrice des indices des partiels


for t=1:nbTrames  %Parcours des trames de F
    tabPeaks=ones(1,nbPeaks);%tab pour connaitre les pics non slectionn par un partiel (si =1 alors pic non slectionn)
    
    
    for l=1:size(Zf,1)
        if(Zi(l,1)~=0)
            if(Zi(l,3)<4)%si le nombre de donnes est inf  4 alors methode de MAQ
                est_x=Zf(l,mod(trameCouranteZ-2,n)+1);
            else
                %Calcul de la prdiction
                x=zeros(1,Zi(l,3));
                for p=1:size(x,2) %reconstitution des dernires donnes du partiel 
                    x(1,p)=Zf(l,mod(trameCouranteZ-2-Zi(l,3)+p,n)+1);
                    if(p==1)
                        verifId=x(1,1);
                    end
                    if(verifId~=x(1,p))
                         verifId=-1;
                    end
                end
       
                
                if(verifId==-1)% si les donnes ne sont pas toutes gales
                    est_x=0;
                    a=arburg(x,floor((size(x,2)-1)/2));
                    for p=2:size(a,2)
                        est_x=est_x - a(p)*x(1,size(x,2)+1-p);
                    end
                else % si les donnes sont toutes gales, pas besoin de calculer une estimation
                    est_x=x(1,1);
                end
            end
            Zf(l,trameCouranteZ)=est_x;
    
            %recherche du peak le plus proche de l'estimation parmis tous ceux disponibles
            indPeak=0;
            deltaF2=deltaF;
            for p=1:nbPeaks
                if(tabPeaks(p)==1 && abs(F(p,t)-est_x)<deltaF2 )
                    indPeak=p;
                    deltaF2=abs(F(p,t)-est_x);
                end
            end
            if(indPeak~=0)
                %mise  jour des Z si pic trouv
                tabPeaks(indPeak)=0;
                Zi(l,2)=dureeActif;
                Zf(l,trameCouranteZ)=F(indPeak,t);
                P(indPeak,t)=Zi(l,1);

            else
                %mise  jour des Z si pic non trouv
                Zi(l,2)=Zi(l,2)-1; 
                if (Zi(l,2)==0)% si partiel a son compteur de dure d'activation  0 alors il meurt.
                    Zi(l,1)=0; 
                    
                end
            end
        end
        %mise  jour du nombre de donnes disponibles
        if(Zi(l,3)<n-1)
            Zi(l,3)=Zi(l,3)+1;
        end
    end
    
    %cration des nouveaux partiels pour les pics non slectionn par un
    %partiel
    for p=1:nbPeaks
        if(tabPeaks(p)==1 && F(p,t)~=0)
            indLibre=0;%recherche d'une ligne de libre dans les matrices de sauvegardes
            for h=1:size(Zi,1)
                if (Zi(h,1)==0)
                   indLibre=h; 
                   break;
                end
            end
            if(indLibre==0) % si pas de ligne de libre alors rajout d'une ligne
               Zf=[Zf;zeros(1,n)];             
               Zf(size(Zf,1),trameCouranteZ)=F(p,t);
               Zi=[Zi;zeros(1,3)];
               Zi(size(Zi,1),1)= tag;
               Zi(size(Zi,1),2)= dureeActif;
               Zi(size(Zi,1),3)=1;
  
            else %si ligne disponible on l'utilise pour le nouveau partiel
                Zf(indLibre,:)=zeros(1,size(Zf,2));
                Zi(indLibre,:)=zeros(1,size(Zi,2));
                Zf(indLibre,trameCouranteZ)=F(p,t);
                Zi(indLibre,1)= tag;
                Zi(indLibre,2)= dureeActif;
                Zi(indLibre,3)=1;
           
            end
            P(p,t)=tag;  
            tag=tag+1;
        end
    end
    trameCouranteZ=mod(trameCouranteZ,size(Zf,2))+1;    
end

