function [A F] = tones2peaks(fundamentalFrequencies, amplitudes, onsets, numberOfHarmonics, durations, sr)
%% generate an artificial peaks representation of a set of tones
%% fundamentalFrequencies : the fundamental frequency of the set of tones (in Hz)
%% amplitudes :  the amplitudes of the set of tones
%% numberOfHarmonics : the number of harmonics of the set of tones
%% durations : the length of the set of tones (in frames)
%% sr : sampling rate (in Hz)
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.

if nargin<1, fundamentalFrequencies = [440 550 880]; end
if nargin<2, amplitudes = [1 1 1]; end
if nargin<3, onsets= [10 15 60]; end
if nargin<4, numberOfHarmonics= [10 10 10]; end
if nargin<5, durations= [20 20 20]; end
if nargin<6, sr=44100; end

A = zeros(sum(numberOfHarmonics), max(onsets+durations));
F = zeros(sum(numberOfHarmonics), max(onsets+durations));
for k=1:length(fundamentalFrequencies)
    [a, f] = tone2peaks(fundamentalFrequencies(k), amplitudes(k), numberOfHarmonics(k), durations(k));
    l=1;
    while F(l, onsets(k)) ~= 0
        l=l+1;
    end
    A(l:l+numberOfHarmonics(k)-1, onsets(k):onsets(k)+durations(k)-1) = a;
    F(l:l+numberOfHarmonics(k)-1, onsets(k):onsets(k)+durations(k)-1) = f;
end

%figure(1);
%imagesc(A);
%figure(2);
%imagesc(F);
%figure(3);
%surf((tones2spectrogram(A, F)), 'LineStyle', 'none');
%P = peaks2partialsDummy(A, F);
%figure(4);
%imagesc(log10(tones2spectrogram(P, F)));