function res = reduced_differentiator_impulse (order)

% see Equation (27) of
% Sylvain Marchand and Philippe Depalle,
% Generalization of the Derivative Analysis Method to Non-Stationary Sinusoidal Modeling,
% In Proceedings of the International Conference on Digital Audio Effects (DAFx),
% pages 281-288, Espoo, Finland, September 2008.
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.

  if nargin < 1
    order = 1023;
  end

  if iseven (order)
    error ('impulse must contain an odd number of points');
  end

  k = (order-1)/2;
  n = [-k:+k];

  h = ((-1).^n) ./ n;
  h(1+k) = 0;  % first derivative filter

  w = hann_window (order);

  res = h .* w;

  % but there are indeed better first derivative filters...
% h1 = +((-1).^n)./((n-0.5).^2);  % better first derivative filter (but +1/2 sample shift)
% h2 = -((-1).^n)./((n+0.5).^2);  % better first derivative filter (but -1/2 sample shift)
  % that should lead to an interesting second derivative filter
% hh = 2*(-1).^(n+1)./(n.^2); hh(1+k) = -pi^2/3;  % second derivative filter
  % in practice, seems to have problems with low frequencies...
  % (so better use the first derivative twice instead...)
% res2 = hh .* w;

end
