function C = fastconv (A, B)
% same as conv (A, B)
% but fast implementation via spectral multiplication
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.

  % length of the result
  L = length(A)+length(B)-1;

  % smallest power of 2 that fits (for faster FFTs)
  %N = 2^nextpow2(L);
  N = L;

  % zero-padding
  A(length(A)+1:N) = 0;
  B(length(B)+1:N) = 0;

  % FFTs
  AA = fft(A);
  BB = fft(B);

  % pointwise multiplication of the spectra
  CC = AA.*BB;

  % inverse FFT
  C = ifft (CC);

  if isreal(A) && isreal(B)
    C = real (C);
  end

  % truncate result
  C = C(1:L);

end
