function [a, mu, phi, omega, psi, delta_t] = reassignment (x, Fs, m)

% see
% Sylvain Marchand and Philippe Depalle,
% Generalization of the Derivative Analysis Method to Non-Stationary Sinusoidal Modeling,
% In Proceedings of the International Conference on Digital Audio Effects (DAFx),
% pages 281-288, Espoo, Finland, September 2008.
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.

  N = length (x);
  % prepare the windows (using the Hann window, two-time differentiable)
  t = time_axis (N, Fs);
  w = hann_window (N, Fs);
  wd = hann_window_d1 (N, Fs);
  tw = t .* w;
  wdd = hann_window_d2 (N, Fs);
  twd = t .* wd;
  % compute the spectra
  Xw = fft (zerophase_signal (w .* x));
  Xwd = fft (zerophase_signal (wd .* x));
  Xtw = fft (zerophase_signal (tw .* x));
  Xwdd = fft (zerophase_signal (wdd .* x));
  Xtwd = fft (zerophase_signal (twd .* x));
  % search for peaks
  if nargin < 3
    % by default, consider the global maximum for the (strictly) positive frequencies
    [v, m] = max (abs (Xw(2:N/2)));
    m = m + 1;
  end
  % frequency
  delta_omega = -imag (Xwd(m) ./ Xw(m));
  base_omega = (m-1)*2*pi*Fs/N;  % bloody MatLab indices...
  omega = base_omega + delta_omega;
  % amplitude modulation
  mu = -real (Xwd(m) ./ Xw(m));
  % time offset (extra)
  delta_t = real (Xtw(m) ./ Xw(m));
  % frequency modulation
  psi = (imag(Xwdd(m)./Xw(m)) - imag((Xwd(m)./Xw(m)).^2)) ./ (real((Xtw(m).*Xwd(m))/(Xw(m)).^2) - real(Xtwd(m)./Xw(m)));
  % then amplitude and phase
  p = Gamma (N, Fs, delta_omega, mu, psi, w);
  phi = angle (Xw(m) ./ p);
  a = abs (Xw(m) ./ p);

end
