function test_case (N, Fs, am, fm)
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.

  test_init;

  tic;

  clip_boundaries = 1;  % 1 to avoid problems at the boundaries... (set it to 0 to see side effects)

  w = hann_window (N, Fs);

  if clip_boundaries
    % then add extra margins (to be clipped)...
    filter = reduced_differentiator_impulse ();
    margin = (length(filter)-1)/2;
    N = N+4*margin;
  end

  a_set = [1];
  phi_set = linspace (-pi, pi, 11);
  phi_set = phi_set(2:10);  % avoid extreme values (bias?!)...
  omega_set = (1:99)*pi*Fs;
  omega_set = omega_set / (length(omega_set)+1);
  omega_set = 0.75 * omega_set;  % restrict to 3/4 Nyquist, since discrete derivatives are severely biased for very high frequencies...

  if am
%   mu_set = [100];
    mu_set = linspace (-100, +100, 5);
  else
    mu_set = [0];
  end

  if fm
%   psi_set = [10000];
    psi_set = linspace (-10000, +10000, 5);
  else
    psi_set = [0];
  end

  % check: with AM, then signal must stay in a reasonable amplitude range
  % check: with FM, omega must stay in the (0, Nyquist) frequency range

  snr_inc = 5;
  snr_set = [-20:snr_inc:100];
  snr_inc_d = 5;
  snr_set_d = [-20:snr_inc_d:100];
  snr_r = snr_inc_d / snr_inc;  % ratio for less evaluations of slow methods...

  try_number = 1;  % number of realizations for the same set of parameters

  init = zeros (length(snr_set), length(a_set)*length(mu_set)*length(phi_set)*length(omega_set)*length(psi_set)*try_number);
  delta_r = init;
  a_ref = init;
  mu_ref = init;
  phi_ref = init;
  omega_ref = init;
  psi_ref = init;
  a_r = init;
  mu_r = init;
  phi_r = init;
  omega_r = init;
  psi_r = init;
  init = zeros (length(snr_set_d), length(a_set)*length(mu_set)*length(phi_set)*length(omega_set)*length(psi_set)*try_number);
  a_td = init;
  mu_td = init;
  phi_td = init;
  omega_td = init;
  psi_td = init;
  a_ed = init;
  mu_ed = init;
  phi_ed = init;
  omega_ed = init;
  psi_ed = init;

  for snr_idx = (1:length(snr_set))
    snr = snr_set(snr_idx);
    idx = 1;
    for a_idx = (1:length(a_set))
      a = a_set(a_idx);
      for mu_idx = (1:length(mu_set))
	mu = mu_set(mu_idx);
	for phi_idx = (1:length(phi_set))
	  phi = phi_set(phi_idx);
	  for omega_idx = (1:length(omega_set))
	    omega = omega_set(omega_idx);
	    for psi_idx = (1:length(psi_set))
	      psi = psi_set(psi_idx);

	      s = signal (N, Fs, a, mu, phi, omega, psi);

	      for try_idx = (1:try_number)

		a_ref(snr_idx,idx) = a;
		mu_ref(snr_idx,idx) = mu;
		phi_ref(snr_idx,idx) = phi;
		omega_ref(snr_idx,idx) = omega;
		psi_ref(snr_idx,idx) = psi;

		n = complex_randn (N);
		n = 10^(-snr/20) * n;
		x = s + n;

	        % "derivatives" of the noise
		nd = discrete_derivative (n, Fs, clip_boundaries);
		ndd = discrete_derivative (nd, Fs, clip_boundaries);

		[nd, margin] = resize_signal (nd, ndd);

		interval = [1+2*margin:N-2*margin];

		Xw = fft (w .* x(interval));
		[v, m] = max (abs (Xw));

		[a_hat, mu_hat, phi_hat, omega_hat, psi_hat, delta_t] = reassignment (x(interval), Fs, m);

		a_r(snr_idx,idx) = a_hat;
		mu_r(snr_idx,idx) = mu_hat;
		phi_r(snr_idx,idx) = phi_hat;
		omega_r(snr_idx,idx) = omega_hat;
		psi_r(snr_idx,idx) = psi_hat;

		delta_r(snr_idx,idx) = delta_t;

		if (mod(snr_idx-1,snr_r) == 0)

		  snr_idx_d = (snr_idx-1) / snr_r+1;  % bloody Matlab indices...

 		  xd = derivative1 (N, Fs, a, mu, phi, omega, psi);
 		  xdd = derivative2 (N, Fs, a, mu, phi, omega, psi);
 		  [a_hat, mu_hat, phi_hat, omega_hat, psi_hat] = derivative (x(interval), xd(interval)+nd, xdd(interval)+ndd, Fs, m);

		  a_td(snr_idx_d,idx) = a_hat;
		  mu_td(snr_idx_d,idx) = mu_hat;
		  phi_td(snr_idx_d,idx) = phi_hat;
		  omega_td(snr_idx_d,idx) = omega_hat;
		  psi_td(snr_idx_d,idx) = psi_hat;

		  xd = discrete_derivative (x, Fs, clip_boundaries);
		  xdd = discrete_derivative (xd, Fs, clip_boundaries);
		  xd = resize_signal (xd, xdd);
		  x = resize_signal (x, xdd);
		  [a_hat, mu_hat, phi_hat, omega_hat, psi_hat] = derivative (x, xd, xdd, Fs, m);

		  a_ed(snr_idx_d,idx) = a_hat;
		  mu_ed(snr_idx_d,idx) = mu_hat;
		  phi_ed(snr_idx_d,idx) = phi_hat;
		  omega_ed(snr_idx_d,idx) = omega_hat;
		  psi_ed(snr_idx_d,idx) = psi_hat;

		end

		idx = idx + 1;
	      end
	    end
	  end
	end
      end
    end
  end

  N = length (interval);

  fprintf ('test was for N=%g\n', N);

  mean (mean (delta_r))
  max (max (abs (delta_r)))

  [a_r_var, mu_r_var, phi_r_var, omega_r_var, psi_r_var] = error_variance (Fs, (a_r-a_ref), (mu_r-mu_ref), (phi_r-phi_ref), (omega_r-omega_ref), (psi_r-psi_ref));
  [a_td_var, mu_td_var, phi_td_var, omega_td_var, psi_td_var] = error_variance (Fs, (a_td-a_ref(1:snr_r:end,:)), (mu_td-mu_ref(1:snr_r:end,:)), (phi_td-phi_ref(1:snr_r:end,:)), (omega_td-omega_ref(1:snr_r:end,:)), (psi_td-psi_ref(1:snr_r:end,:)));
  [a_ed_var, mu_ed_var, phi_ed_var, omega_ed_var, psi_ed_var] = error_variance (Fs, (a_ed-a_ref(1:snr_r:end,:)), (mu_ed-mu_ref(1:snr_r:end,:)), (phi_ed-phi_ref(1:snr_r:end,:)), (omega_ed-omega_ref(1:snr_r:end,:)), (psi_ed-psi_ref(1:snr_r:end,:)));

  sigma = 10.^(-snr_set/20);

  [a_crb, mu_crb, phi_crb, omega_crb, psi_crb] = crb (sigma, a, mu_set/Fs, psi_set/(Fs^2), N);

  close all;

  % amplitude
  plot (snr_set, log10(a_r_var), 'ko-', snr_set_d, log10(a_td_var), 'kx-', snr_set_d, log10(a_ed_var), 'k+-', snr_set, log10(a_crb), 'k-');
  title ('estimation of the amplitude', 'FontName', 'Times-Roman', 'FontSize', 20);
  xlabel ('signal-to-noise ratio (dB)', 'FontName', 'Times-Roman', 'FontSize', 20);
  ylabel ('variance of the error (log10 scale)', 'FontName', 'Times-Roman', 'FontSize', 20);
  legend ('R', 'TD', 'ED', 'CRB');

  if am
    if fm
      print -deps 'comp-a-amfm.eps';
    else
      print -deps 'comp-a-am.eps';
    end
  else
    if fm
      print -deps 'comp-a-fm.eps';
    else
      print -deps 'comp-a.eps';
    end
  end

  figure;

  % amplitude modulation
  plot (snr_set, log10(mu_r_var), 'ko-', snr_set_d,log10(mu_td_var), 'kx-', snr_set_d,log10(mu_ed_var), 'k+-', snr_set,log10(mu_crb), 'k-');
  title ('estimation of the amplitude modulation', 'FontName', 'Times-Roman', 'FontSize', 20);
  xlabel ('signal-to-noise ratio (dB)', 'FontName', 'Times-Roman', 'FontSize', 20);
  ylabel ('variance of the error (log10 scale)', 'FontName', 'Times-Roman', 'FontSize', 20);
  legend ('R', 'TD', 'ED', 'CRB');

  if am
    if fm
      print -deps 'comp-mu-amfm.eps';
    else
      print -deps 'comp-mu-am.eps';
    end
  else
    if fm
      print -deps 'comp-mu-fm.eps';
    else
      print -deps 'comp-mu.eps';
    end
  end

  figure;

  % phase
  plot (snr_set,log10(phi_r_var), 'ko-', snr_set_d, log10(phi_td_var), 'kx-', snr_set_d, log10(phi_ed_var), 'k+-', snr_set, log10(phi_crb), 'k-');
  title ('estimation of the phase', 'FontName', 'Times-Roman', 'FontSize', 20);
  xlabel ('signal-to-noise ratio (dB)', 'FontName', 'Times-Roman', 'FontSize', 20);
  ylabel ('variance of the error (log10 scale)', 'FontName', 'Times-Roman', 'FontSize', 20);
  legend ('R', 'TD', 'ED', 'CRB');

  if am
    if fm
      print -deps 'comp-phi-amfm.eps';
    else
      print -deps 'comp-phi-am.eps';
    end
  else
    if fm
      print -deps 'comp-phi-fm.eps';
    else
      print -deps 'comp-phi.eps';
    end
  end

  figure;

  % frequency
  plot (snr_set, log10(omega_r_var), 'ko-', snr_set_d, log10(omega_td_var), 'kx-', snr_set_d,log10(omega_ed_var), 'k+-', snr_set, log10(omega_crb), 'k-');
  title ('estimation of the frequency', 'FontName', 'Times-Roman', 'FontSize', 20);
  xlabel ('signal-to-noise ratio (dB)', 'FontName', 'Times-Roman', 'FontSize', 20);
  ylabel ('variance of the error (log10 scale)', 'FontName', 'Times-Roman', 'FontSize', 20);
  legend ('R', 'TD', 'ED', 'CRB');

  if am
    if fm
      print -deps 'comp-omega-amfm.eps';
    else
      print -deps 'comp-omega-am.eps';
    end
  else
    if fm
      print -deps 'comp-omega-fm.eps';
    else
      print -deps 'comp-omega.eps';
    end
  end

  figure;

  % frequency modulation
  plot (snr_set, log10(psi_r_var), 'ko-', snr_set_d, log10(psi_td_var), 'kx-', snr_set_d, log10(psi_ed_var), 'k+-', snr_set,log10(psi_crb), 'k-');
  title ('estimation of the frequency modulation', 'FontName', 'Times-Roman', 'FontSize', 20);
  xlabel ('signal-to-noise ratio (dB)', 'FontName', 'Times-Roman', 'FontSize', 20);
  ylabel ('variance of the error (log10 scale)', 'FontName', 'Times-Roman', 'FontSize', 20);
  legend ('R', 'TD', 'ED', 'CRB');

  if am
    if fm
      print -deps 'comp-psi-amfm.eps';
    else
      print -deps 'comp-psi-am.eps';
    end
  else
    if fm
      print -deps 'comp-psi-fm.eps';
    else
      print -deps 'comp-psi.eps';
    end
  end

  toc;

end
