function R = peaksCompute(S, ap)
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.

nbBins = ap.peaks.nbBins;

step = fix(size(S, 1)/nbBins);

R=zeros(ap.peaks.nbPeaks*2, size(S, 2));
for k=1:size(S, 2)
    s=S(:, k);

    ind=zeros(nbBins, 1);
    minValue=zeros(nbBins, 1);
    for l=0:nbBins-1
        [value i] = max(s(l*step+1:(l+1)*step));
        ind(l+1) = l*step+i(1);
        minValue(l+1) = min(s(l*step+1:(l+1)*step));
    end

    % select only the prominent peaks
    decisionVector = 20*log10(s(ind)./minValue);
    threshold = min(decisionVector)+(max(decisionVector)-min(decisionVector))/ap.peaks.thresholdLevel;
    ind(decisionVector<threshold)=0;
    ind = ind(ind>0);
    vt = [0; s; 0];
    ind(vt(ind+1)<vt(ind))=0;
    ind(vt(ind+1)<vt(ind+2))=0;
    ind = ind(ind>0);
    vind = s(ind);
    
    %select only the nbPeaks highest peaks
    if length(ind) > ap.peaks.nbPeaks
     [vind vindex] = sort(vind, 'descend');
     ind = ind(vindex);
     
     vind = vind(1:ap.peaks.nbPeaks);
     ind = ind(1:ap.peaks.nbPeaks);
    elseif length(ind) < ap.peaks.nbPeaks
        vind = [vind; zeros(ap.peaks.nbPeaks-length(ind), 1)];
    ind = [ind; zeros(ap.peaks.nbPeaks-length(ind), 1)];
    end
    tmp  = [vind; ind];
    R(:, k) = tmp;
end