function ap = getAnalysisParameters(sr)
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.

if nargin<1
    sr=44100;
end

%% assumed sampling rate
ap.sr = sr;

%% time domain input signal massaging
ap.preEmphasis = .97;
ap.dcBlock = .999;

%% spectral analysis parameters
ap.frameSize = 2^(nextpow2(0.04*sr));
ap.hopSize = ap.frameSize/4;
ap.textureWindowSize = 20;
ap.eventWindowSize = 100;
ap.horizonWindowSize = 500;

%% features parameters

% mfcc
ap.mfcc.numcep = 13;
ap.mfcc.minfreq = 30;
ap.mfcc.maxfreq = ceil(ap.sr/2); % 4000;
ap.mfcc.nbands = 40;
ap.mfcc.bwidth = 1.0;
ap.mfcc.dcttype = 2;
ap.mfcc.fbtype = 'mel';
ap.mfcc.sumpower = 1;
ap.mfcc.lifterexp = .6;

% ar envelope
ap.ar.order = 100;
ap.ar.envelopeLength = ap.frameSize/2;

% true envelope
ap.te.order = 100;
ap.te.nbIteration = 40;
ap.te.deltaThreshold = 10;
ap.te.envelopeLength = ap.frameSize/2;

% peaks
ap.peaks.nbPeaks = 30;
ap.peaks.nbBins = ap.frameSize/32;
ap.peaks.thresholdLevel = 6;


% long term modelling
ap.lt.nbBins = 64-2;
ap.lt.nbBands = 20;
ap.lt.order = 8;
ap.lt.polynomOrder = 9;
ap.lt.vectorLength = ap.lt.order*2;
ap.lt.selectorStart = 2;
ap.lt.numcep = ap.mfcc.numcep;

