function conf = getConfig(databaseId, ...
    featuresId, featuresDataPercentage, ...
    degradationId, degradationLevel, degradationSymmetry, ...
    integrationId, ...
    classificationId, similarityDataPercentage, similarityDataSplit, similarityDataTruncate, ...
    sendMail, verbose, comment)
%% Set the configuration parameters for the whole process
% features computation, similarity computation and performances computation
% databseId = 1 (iowa), 2 (rwc), 3 (solos), 4 (poly inclusive), 5
% (polyExclusive)
% featuresDataPercentage : 0 debug <0 truncate by files >0 truncate within
% files
% degradationId = 1 'noiseAddition', 2 'lowPassFiltering',  3 'lossyEncoding',
% 4 'spectralDigging', 5 'sinusoidalSynthesis'}
% degradationLevel from 0 to 1
% integrationId = -1 'file', 0 'none', 1 'texture", 2 'onset'
% similarityDataPercentage : percentage of the selection to be considered for similarity
% computation from 0 to 1
% similarityDataSplit : number of sub matrices considered for similarity computation
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.


if nargin<1
    databaseId = 1;
end
if nargin<2
    featuresId = 0;
end
if nargin<3
    featuresDataPercentage = 0;
end
if nargin<4
    degradationId = 0;
end
if nargin<5
    degradationLevel = 0;
end
if nargin<6
    degradationSymmetry = 1;
end
if nargin<7
    integrationId = 1;
end
if nargin<8
    classificationId = 0;
end
if nargin<9
    similarityDataPercentage = 1;
end
if nargin<10
    similarityDataSplit = 1;
end
if nargin<11
    similarityDataTruncate = 0;
end
if nargin<12
    sendMail = 0;
end
if nargin<13
    verbose = 0;
end
if nargin<14
    comment = '';
end

conf.databaseId = databaseId;
conf.database = getDatabaseType(databaseId);

if featuresId<0 && featuresId>-5
    [featuresIdFromExperiment, classificationIdFromExperiment] = getFCperExperiment(abs(featuresId));
    conf.featuresId = featuresIdFromExperiment;
else
    featuresIdFromExperiment = [];
    classificationIdFromExperiment = [];
    conf.featuresId = featuresId;
end

conf.features = getFeatureType(conf.featuresId);
conf.featuresDataPercentage = featuresDataPercentage;

conf.degradation = getDegradationType(degradationId);
conf.degradationId = degradationId;
conf.degradationLevel = degradationLevel;
conf.degradationSymmetry = degradationSymmetry;
conf.integration = getIntegrationType(integrationId);
conf.integrationId = integrationId;

if sum(classificationId) == 0 && ~isempty(classificationIdFromExperiment)
    classificationId = classificationIdFromExperiment;
end

conf.classification = getClassificationType(classificationId);
conf.classificationId = classificationId;
% 1 Gmm else Svm
conf.classifierType = 0;

conf.rootPath = '/data/lagrange/';
conf.dataPath = [conf.rootPath 'plato/' char(conf.database) '/']; %% , '%d-'
conf.fileName = [char(conf.database) '_' sprintf('%1d-', featuresId)  '_' num2str(conf.featuresDataPercentage) '_' char(conf.degradation) '_' num2str(conf.degradationLevel) '_' char(conf.integration)];
conf.featureFileName = [conf.rootPath 'features/' conf.fileName '.mat'];
conf.onsetCurveFileName = [conf.rootPath 'onsetCurves/johndoe_onsetCurve.dat'];

conf.similarityDataPercentage = similarityDataPercentage;
conf.similarityDataSplitComputation = similarityDataSplit;
conf.similarityDataTruncate = similarityDataTruncate;
conf.distanceFileName = [conf.rootPath 'distance/' conf.fileName '_' num2str(similarityDataPercentage) '_' num2str(degradationSymmetry) '.mat'];

conf.performanceSr = 1000;

conf.performanceFileName = [conf.rootPath 'performance/' conf.fileName '_' num2str(similarityDataPercentage) '_' num2str(degradationSymmetry) '_' num2str(similarityDataTruncate)];

% degradation levels
conf.defaultDegradationLevels = [...
    [.25 1/16]; ... % noise addition 12 dB 24 dB
    [7/8 3/4]; ... % lowpass  5k 11k
    [1 .6]; ... % lossy encoding 32 96 kps
    [.5 .25]; ... % raz 50 25 %
    [.5 .25]; ... % keep 50 75 %
    ];

% smaller degradation
conf.defaultDegradationLevels = [...
    [1/16 1/32]; ... % noise addition 
    [3/4 7/8]; ... % lowpass 
    [.6 .3]; ... % lossy encoding 
    [.25 .125]; ... % raz 
    [.25 .125]; ... % keep 
    ];

conf.sendMail = sendMail;
conf.email = 'lagrange@enst.fr';
conf.verbose=verbose;
conf.comment=comment;
